@@ -11,6 +11,7 @@ Thanks to Werner van Geit for an initial version of a python wrapper for jnml.
 
 from __future__ import absolute_import
 from __future__ import print_function
+from __future__ import unicode_literals
 import os
 import sys
 import subprocess
@@ -221,6 +222,13 @@ def parse_arguments():
             help=('Load a NeuroML file, and convert it to a graph using\n'
                   'GraphViz. Detail is set by level (1, 2, etc.)')
             )
+    mut_exc_opts.add_argument(
+            '-matrix',
+            metavar=('level'),
+            nargs=1,
+            help=('Load a NeuroML file, and convert it to a matrix displaying\n'
+                  'connectivity. Detail is set by level (1, 2, etc.)')
+            )
     mut_exc_opts.add_argument(
             '-validate',
             action='store_true',
@@ -432,6 +440,125 @@ def quick_summary(nml2_doc):
             info+=']\n'
     return info
 
+def summary(nml2_doc=None, verbose=False):
+    '''
+    Based on nml2_doc.summary(show_includes=False)
+    '''
+
+    if '-v' in sys.argv:
+        verbose = True
+        
+    if nml2_doc is None:
+        nml2_file_name = sys.argv[1]
+        nml2_doc = read_neuroml2_file(nml2_file_name, include_includes=verbose)
+        
+    
+    info = nml2_doc.summary(show_includes=False)
+    
+    if verbose:
+        cell_info_str = ''
+        for cell in nml2_doc.cells:
+            cell_info_str+=cell_info(cell)+'*\n'
+        lines = info.split('\n')
+        info = ''
+        still_to_add = False
+        for line in lines:
+            if 'Cell: ' in line:
+                still_to_add = True
+                pass
+            elif 'Network: ' in line:
+                still_to_add = False
+                if len(cell_info_str)>0:
+                    info+='%s'%cell_info_str
+                info+='%s\n'%line
+            else:
+                if still_to_add and '******' in line:
+                    if len(cell_info_str)>0:
+                        info+='%s'%cell_info_str
+                info+='%s\n'%line
+    print(info)
+
+
+def cells_info(nml_file_name):
+    
+    from neuroml.loaders import read_neuroml2_file
+    nml_doc = read_neuroml2_file(nml_file_name,include_includes=True, verbose=False, optimized=True)
+    
+    info = ''
+    info+='Extracting information on %i cells in %s'%(len(nml_doc.cells), nml_file_name)
+    
+    for cell in nml_doc.cells:
+        info+=cell_info(cell)
+        
+    return info
+    
+def cell_info(cell):
+    
+    info = ''
+    prefix = '*  '
+    info+=prefix+'Cell: %s\n'%cell.id
+    tot_length = 0
+    tot_area = 0
+    for seg in cell.morphology.segments:
+        
+        info+=(prefix+'  %s\n'%seg)
+        dist = seg.distal
+        prox = seg.proximal
+        parent_id = seg.parent.segments if seg.parent else 'None (root segment)'
+        length = cell.get_segment_length(seg.id)
+        info+=(prefix+'    Parent segment: %s\n'%(parent_id))
+        info+=(prefix+'    %s -> %s; seg length: %s um\n'%(prox, dist, length))
+        
+        tot_length+=length
+        area = cell.get_segment_surface_area(seg.id)
+        volume = cell.get_segment_volume(seg.id) 
+        tot_area+= area
+        info+=(prefix+'    Surface area: %s um2, volume: %s um3\n'%(area, volume))
+    numseg = len(cell.morphology.segments)
+    info+=(prefix+'  Total length of %i segment%s: %s um; total area: %s um2\n'%(numseg, 's' if numseg>1 else '', tot_length, tot_area))
+    
+    info+=(prefix+'\n')
+    
+    for sg in cell.morphology.segment_groups:
+        segs = cell.get_all_segments_in_group(sg.id)
+        info+=(prefix+'  %s;\tcontains %i segment%s\n'%(str(sg).replace(', ', ',\t'), len(segs), ', id: %s'%segs[0] if len(segs)==1 else 's in total'))
+        
+    if len(cell.morphology.segment_groups)>0:
+        info+=(prefix+'\n')
+        
+    seg_info = cell.get_segment_ids_vs_segments()
+    for cd in cell.biophysical_properties.membrane_properties.channel_densities:
+        #print dir(cd)
+        group = cd.segment_groups if cd.segment_groups else 'all'
+        info+=(prefix+'  Channel density: %s on %s;\tconductance of %s through ion chan %s with ion %s, erev: %s\n'%(cd.id, group, cd.cond_density, cd.ion_channel, cd.ion, cd.erev))
+        segs = cell.get_all_segments_in_group(group)
+        for seg_id in segs:
+            seg = seg_info[seg_id]
+            
+            cond_dens_si = get_value_in_si(cd.cond_density)
+            surface_area_si = get_value_in_si('%s um2'%cell.get_segment_surface_area(seg_id))
+            cond_si = cond_dens_si * surface_area_si
+            cond_pS = convert_to_units('%sS'%cond_si, 'pS')
+            info+=(prefix+'    Channel is on %s,\ttotal conductance: %s S_per_m2 x %s m2 = %s S (%s pS)\n'%(seg, cond_dens_si, surface_area_si, cond_si, cond_pS))
+            
+    if len(cell.biophysical_properties.membrane_properties.channel_densities)>0:
+        info+=(prefix+'\n')
+        
+    for sc in cell.biophysical_properties.membrane_properties.specific_capacitances:
+        group = sc.segment_groups if sc.segment_groups else 'all'
+        info+=(prefix+'  Specific capacitance on %s: %s\n'%(group, sc.value))
+        segs = cell.get_all_segments_in_group(group)
+        for seg_id in segs:
+            seg = seg_info[seg_id]
+            spec_cap_si = get_value_in_si(sc.value)
+            surface_area_si = get_value_in_si('%s um2'%cell.get_segment_surface_area(seg_id))  
+            cap_si = spec_cap_si * surface_area_si
+            cap_pF = convert_to_units('%sF'%cap_si, 'pF')
+            info+=(prefix+'    Capacitance of %s,\ttotal capacitance: %s F_per_m2 x %s m2 = %s F (%s pF)\n'%(seg, spec_cap_si, surface_area_si, cap_si, cap_pF))
+
+            
+    return info
+            
 
 def write_neuroml2_file(nml2_doc, nml2_file_name, validate=True, 
                         verbose_validate=False):
@@ -442,7 +569,6 @@ def write_neuroml2_file(nml2_doc, nml2_file_name, validate=True,
         validate_neuroml2(nml2_file_name, verbose_validate)
         
         
-
 def read_lems_file(lems_file_name, include_includes=False, fail_on_missing_includes=False, debug=False):
     
     if not os.path.isfile(lems_file_name):
@@ -694,11 +820,65 @@ def run_lems_with_jneuroml_netpyne(lems_file_name,
                                  t_run = t_run,
                                  plot = plot, 
                                  show_plot_already = show_plot_already, 
-                                 simulator = 'jNeuroML_NEURON',
+                                 simulator = 'jNeuroML_NetPyNE',
                                  reload_events = reload_events,
                                  remove_dat_files_after_load = cleanup)
     else:
         return True
+
+
+def run_lems_with_jneuroml_brian2(lems_file_name,
+                                  paths_to_include = [], 
+                                  max_memory = DEFAULTS['default_java_max_memory'], 
+                                  skip_run = False,
+                                  nogui = False, 
+                                  load_saved_data = False, 
+                                  reload_events = False,
+                                  plot = False, 
+                                  show_plot_already = True, 
+                                  exec_in_dir = ".",
+                                  verbose = DEFAULTS['v'],
+                                  exit_on_fail = True,
+                                  cleanup = False):
+                                      
+    print_comment("Loading LEMS file: %s and running with jNeuroML_Brian2" \
+                  % lems_file_name, verbose)
+                  
+    post_args = " -brian2"
+    
+    post_args += gui_string(nogui)
+    post_args += include_string(paths_to_include)
+    
+    t_run = datetime.now()
+    if skip_run:
+      success = True
+    else:
+      success = run_jneuroml("", 
+                           lems_file_name, 
+                           post_args, 
+                           max_memory = max_memory, 
+                           exec_in_dir = exec_in_dir, 
+                           verbose = verbose, 
+                           exit_on_fail = exit_on_fail)
+      print('oooooo')
+      import LEMS_SimFNTest_brian2
+      print('eeee')
+    
+    if not success: 
+        return False
+    
+    if load_saved_data:
+        return reload_saved_data(lems_file_name, 
+                                 base_dir = exec_in_dir,
+                                 t_run = t_run,
+                                 plot = plot, 
+                                 show_plot_already = show_plot_already, 
+                                 simulator = 'jNeuroML_Brian2',
+                                 reload_events = reload_events,
+                                 remove_dat_files_after_load = cleanup)
+    else:
+        return True
+    
     
     
 def reload_saved_data(lems_file_name, 
@@ -964,14 +1144,16 @@ def evaluate_arguments(args):
         
     elif args.graph:
         
-        
         from neuromllite.GraphVizHandler import GraphVizHandler, engines
         engine = 'dot'
         try:
             level = int(args.graph[0])
         except:
             engine = engines[args.graph[0][-1:]]
-            level = int(args.graph[0][:-1])
+            level_int = args.graph[0][:-1]
+            level_int = level_int.replace('min','-')
+            level_int = level_int.replace('m','-')
+            level = int(level_int)
         
         print_comment('Converting %s to graphical form, level %i, engine %s'%(args.lems_file,level,engine))
         
@@ -989,6 +1171,29 @@ def evaluate_arguments(args):
 
         exit()
         
+    elif args.matrix:
+        
+        
+        from neuromllite.MatrixHandler import MatrixHandler
+        
+        level = int(args.matrix[0])
+        
+        print_comment('Converting %s to matrix form, level %i'%(args.lems_file,level))
+        
+        from neuroml.hdf5.NeuroMLXMLParser import NeuroMLXMLParser
+
+        handler = MatrixHandler(level=level, nl_network=None)
+
+        currParser = NeuroMLXMLParser(handler)
+
+        currParser.parse(args.lems_file)
+
+        handler.finalise_document()
+
+        print_comment("Done with MatrixHandler...")
+
+        exit()
+        
     elif args.validate:
         pre_args = "-validate"
         exit_on_fail = True
@@ -1138,12 +1343,17 @@ def execute_command_in_dir_with_realtime_output(command, directory, verbose=DEFA
                 print_comment(line, end="")
         p.wait() # wait for the subprocess to exit
     except KeyboardInterrupt as e:
+        
+        print_comment_v('*** Command interrupted: \n       %s'%command)
         if p:
             p.kill()
         #return True
         raise e
 
-    return True
+    if not p.returncode==0:
+        print_comment_v('*** Problem running command (return code: %s): \n       %s'%(p.returncode,command))
+    
+    return p.returncode==0
 
 
 def execute_command_in_dir(command, directory, verbose=DEFAULTS['v'], 
